/*
 * This file is a part of Luminance HDR package.
 * ----------------------------------------------------------------------
 * Copyright (C) 2013 Franco Comida
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHo ANY WARRANTY; witho even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ----------------------------------------------------------------------
 */

#include <Libpfs/colorspace/yuv.h>
#include <Libpfs/colorspace/rgb.h>

namespace pfs {
namespace colorspace {

//! \brief Basic matrices for the RGB <-> YUV conversion
const float rgb2yuvMat[3][3] =
{ { 0.299f, 0.587f, 0.144f },
  { -0.299f, -0.587f, 0.886f },
  { 0.701f, -0.587f, -0.114f } };

const float yuv2rgbMat[3][3] =
{ { 0.97087f, -0.02913f, 1.0f },
  { 0.97087f, -0.22333f, -0.50937f },
  { 0.97087f, 0.97087f, 0.0f } };

void ConvertRGB2YUV::operator()(float i1, float i2, float i3,
                                float& o1, float& o2, float& o3) const
{
    o1 = rgb2yuvMat[0][0]*i1 + rgb2yuvMat[0][1]*i2 + rgb2yuvMat[0][2]*i3;
    o2 = rgb2yuvMat[1][0]*i1 + rgb2yuvMat[1][1]*i2 + rgb2yuvMat[1][2]*i3;
    o3 = rgb2yuvMat[2][0]*i1 + rgb2yuvMat[2][1]*i2 + rgb2yuvMat[2][2]*i3;
}

void ConvertYUV2RGB::operator()(float i1, float i2, float i3,
                                float& o1, float& o2, float& o3) const
{
    o1 = yuv2rgbMat[0][0]*i1 + yuv2rgbMat[0][1]*i2 + yuv2rgbMat[0][2]*i3;
    o2 = yuv2rgbMat[1][0]*i1 + yuv2rgbMat[1][1]*i2 + yuv2rgbMat[1][2]*i3;
    o3 = yuv2rgbMat[2][0]*i1 + yuv2rgbMat[2][1]*i2 + yuv2rgbMat[2][2]*i3;
}

}   // colorspace
}   // pfs
